#include "../headers/etc/StaticData.h"
#include "../headers/etc/Parameter.h"
#include "../headers/etc/Typedef.h"

#include "../headers/Model/LanguageModel.h"

#include "../headers/DataStructure/Phrase.h"

using namespace std;

// t[YꃂfRXgvZ
float LanguageModel::getLMScore(const Phrase& phrase) {
  return getLMScore(phrase.getLMWordIDArray());
}

float LanguageModel::getLMScoreForSentenceEnd(){
  copy_backward(idArray, idArray+historySize, idArray+historySize+1);
  *(idArray+historySize+1) = Vocab_None;
  *idArray = seTagID;

  return getNGramScore(idArray);
}

// ꃂfRXgvZfor sentence
float LanguageModel::getLMScore(LMWordIDArray& sentence, const unsigned sentenceSize, const float threshold) const {
  // idx     : size-3   size-2   size-1     size
  // sentence: ...,     ssTagID, Vocab_None [eoa]

  unsigned start = sentenceSize - 3;  
  unsigned end   = sentenceSize - 1;
  
  // RXgvZ
  float score = 0.0F;

  do{
    score += getNGramScore( &sentence[start] );
    /*
      const unsigned* pos = &sentence[start];
      cerr << "p( " << getWord(*pos) << " | ";
      for(unsigned i=1; i<nGramOrder && (*(pos+i)) != Vocab_None; i++)
        cerr << getWord(*(pos+i)) << " ";
      cerr << ") = ";
      cerr << getNGramScore(pos) << endl;
      */
    
    // if( score < threshold ) break;

    if( (end-start) >= nGramOrder ) sentence[--end] = Vocab_None;
  }while( 0 != start-- );

  return score;
}

// ꃂfRXgvZ
float LanguageModel::getLMScore(const LMWordIDArray& newPhrase, const LMWordIDArray& history, const float threshold, bool debug){
  copy(newPhrase.begin(), newPhrase.end(), idArray);
  copy(history.begin(), history.end(), idArray+newPhrase.size());

  unsigned start = newPhrase.size() - 1;
  unsigned end   = newPhrase.size() + history.size();
  idArray[end]     = Vocab_None;

  if(debug){
    cerr << "new phrase: " ;
    for(unsigned i=0; i<newPhrase.size(); i++)
      cerr << getWord(newPhrase[i]) << "_";
    cerr << endl;
    
    cerr << "history: ";
    for(unsigned i=0; i<history.size(); i++)
      cerr << getWord(history[i]) << "_";
    cerr << endl;
    
    cerr << "idArray: ";
    for(unsigned i=0; i<end; i++)
      cerr << getWord(idArray[i]) << "_";
    cerr << endl;
  }

  // RXgvZ
  float score = 0.0F;
  do{
    score += getNGramScore( &idArray[start] );
    if(score < threshold) return score; // early pruning

    if(debug){
      cerr << "LM for: ";
      for(unsigned i=start; i<end; i++)
	cerr << getWord(idArray[i]) << "_";
      cerr << endl;
    }

    if( (end-start) >= nGramOrder ) idArray[--end] = Vocab_None;
  }while( 0 != start-- );

  historySize = end;

  return score;
}

// o萔
const float   LanguageModel::log10e         = (float)log10(exp(1.0));
const float   LanguageModel::unkWordPenalty = -100.0F;
string        LanguageModel::ssTag = "<s>";
string        LanguageModel::seTag = "</s>";
LMWordID      LanguageModel::seTagID = 0;
LMWordID      LanguageModel::ssTagID = 0;
LMWordIDArray LanguageModel::seTagIDArray;
LMWordIDArray LanguageModel::ssTagIDArray;

