#include <iostream>
#include <fstream>
#include <sys/time.h>
#include <ctime>

#include "../headers/etc/StaticData.h"
#include "../headers/etc/Parameter.h"

#include "../headers/Model/LanguageModel.h"
#include "../headers/Model/PhraseTable.h"
#include "../headers/Model/LatticeManager.h"

#include "../headers/DataStructure/Sentence.h"
#include "../headers/DataStructure/TriangleTable.h"
#include "../headers/DataStructure/Coverage.h"

#include "../headers/DP_BeamSearch/Translator.h"


using namespace std;

void printVersion(){
  cerr << "Khafra --a phrase-based DP beam search decoder--" << endl;
  cerr << "  version " << version << endl;
  cerr << "  release date: " << releaseDate << endl;
  cerr << "  written by Mitsuru KOSHIKAWA(MIBEL at the Univ. of Tsukuba)" << endl;
}

unsigned Coverage::size     = 0;
unsigned Coverage::capacity = 0;

int main(const int argc, const char** argv){
  printVersion();

  StaticData::getInstance().initialize(argc, argv);
  const Parameter& param = StaticData::getInstance().getParameter();
  CoverageHypothesis::setBeamWidth(param.getBeamWidth());
  CoverageHypothesis::setBeamThreshold(param.getBeamThreshold());

  if(StaticData::getInstance().getParameter().getVerbose() >= 3)
    StaticData::getInstance().getPhraseTable().print();


  const string& inputFileName = StaticData::getInstance().getParameter().getInputFileName();
  ifstream inputFile(inputFileName.c_str());
  istream& inputStream = inputFile.fail() ? std::cin : inputFile;

  unsigned num = 0;
  string   line;
  double   time = 0.0;

  while(getline(inputStream, line) != NULL){
    Sentence srcSentence(line);         // string to Sentence class

    cerr << "<Sentence No." << ++num << ">" << endl;
    cerr << "SRC: " << srcSentence << " (consists of " << srcSentence.size() << " tokens )"<< endl;

    Coverage::setSize(srcSentence.size());                      //
    LatticeManager ltcManager(srcSentence);                     // make lattice for the input sentence
    CoverageHypothesis::setPtLatticeManeger(&ltcManager);
    LexicalHypothesis::resetHypoID();

    timeval startTimeSentence;  gettimeofday(&startTimeSentence, NULL); // calcurate only translation time(not including {pre,post}process)

    // Execute DP Beam Search
    Translator translator(ltcManager);
    string translation = translator.translate();

    timeval endTimeSentence; gettimeofday(&endTimeSentence, NULL);

    double timeForThisSentence = diffTimeVal(startTimeSentence, endTimeSentence);
    time += timeForThisSentence;

    cerr << "TGT: " << strip(translation) << endl;
    cerr << "Hyp.Score: " << translator.getBestScore() << endl;
    cerr << "#Lex.Hyp.: " << translator.getExpandedLexHypNum() << endl;
    cerr << "Tlt.Time : " << timeForThisSentence << endl;

    cout << strip(translation) << endl;

    cerr << endl;
  }

  cerr << "translated " << num << " sentence in " << time << " [sec]" << endl;

  // delete the tables of LM,TM,RM
  StaticData::deleteInstance();

  return 0;
}
