#ifndef _LEX_HYPOTHESIS_
#define _LEX_HYPOTHESIS_

#include "../etc/Typedef.h"

#include "../Model/LanguageModel.h"

#include "../DataStructure/Coverage.h"
#include "../DataStructure/ObjectPool.h"

class LexicalHypothesis;
FixedObjectPool<LexicalHypothesis> lexHypPool;

class LexicalHypothesis{
 private:
  static unsigned long hypoNum;
  unsigned long id;

  float score;
  float futureScore;

  const Coverage* ptCoverage;

  // recombinep
  const PhrasePair*   ptLastPhrasePair;
  LMWordIDArray       lmHistory;
  int                 lastCoveredSrcPos;
  const void*         lmState;

  // back tracep
  const LexicalHypothesis*     ptPrevHypothesis;

 public:
  static void resetHypoID() { hypoNum = 0; }
  static const unsigned long& getCount() { return hypoNum; }

  explicit LexicalHypothesis(const Coverage* ptCoverage=NULL,
      const float score=0.0F,
      const float futureScore=0.0F,
      const PhrasePair* ptLastPhrasePair = NULL,
      const LMWordIDArray& lmHistory = LMWordIDArray(4, LanguageModel::ssTagID),
      const void* lmState = NULL,
      const int lastCoveredSrcPos = -1,
      const LexicalHypothesis* ptPrevHypothesis = NULL)
  : id(hypoNum++), score(score), futureScore(futureScore), ptCoverage(ptCoverage),
    ptLastPhrasePair(ptLastPhrasePair), lmHistory(lmHistory), lmState(lmState),
    lastCoveredSrcPos(lastCoveredSrcPos), ptPrevHypothesis(ptPrevHypothesis)
  {}

  explicit LexicalHypothesis(const Coverage* ptCoverage,
      const float score,
      const float futureScore,
      const PhrasePair* ptLastPhrasePair,
      const LMWordID* lmHistoryStart,
      const LMWordID* lmHistoryEnd,
      const void* lmState, 
      const int lastCoveredSrcPos,
      const LexicalHypothesis* ptPrevHypothesis)
  : id(hypoNum++), score(score), futureScore(futureScore), ptCoverage(ptCoverage),
    ptLastPhrasePair(ptLastPhrasePair), lmHistory(lmHistoryStart, lmHistoryEnd), lmState(lmState),
    lastCoveredSrcPos(lastCoveredSrcPos), ptPrevHypothesis(ptPrevHypothesis)
    {}

  LexicalHypothesis(const LexicalHypothesis& lexHypo)
    : id(lexHypo.getID()), score(lexHypo.getScore()), futureScore(lexHypo.getFutureScore()),
      ptCoverage(&lexHypo.getCoverage()),
    ptLastPhrasePair(lexHypo.getLastPhrasePair()), lmHistory(lexHypo.getLMHistory()),
    lmState(lexHypo.getLMState()), 
      lastCoveredSrcPos(lexHypo.getLastCoveredSrcPos()), ptPrevHypothesis(lexHypo.getPrevLexicalHypothesis())
    {}

  LexicalHypothesis& operator=(const LexicalHypothesis& rhs){
    id = rhs.getID();
    score = rhs.getScore();
    futureScore = rhs.getFutureScore();
    ptCoverage = &rhs.getCoverage();
    ptLastPhrasePair = rhs.getLastPhrasePair();
    lmHistory = rhs.getLMHistory();
    lmState = rhs.getLMState();
    lastCoveredSrcPos = rhs.getLastCoveredSrcPos();
    ptPrevHypothesis = rhs.getPrevLexicalHypothesis();

    return *this;
  }

  void assign(const Coverage* ptCoverage,
               const float score,
               const float futureScore,
               const PhrasePair* ptLastPhrasePair,
               const LMWordID* lmHistoryStart,
               const LMWordID* lmHistoryEnd,
	       const void* lmState,
               const int lastCoveredSrcPos,
               const LexicalHypothesis* ptPrevHypothesis)
  {
    this->ptCoverage = ptCoverage;
    this->score = score;
    this->futureScore = futureScore;
    this->ptLastPhrasePair = ptLastPhrasePair;
    this->lmHistory.assign(lmHistoryStart, lmHistoryEnd);
    this->lmState = lmState;
    this->lastCoveredSrcPos = lastCoveredSrcPos;
    this->ptPrevHypothesis = ptPrevHypothesis;
  }

  void assign(const LexicalHypothesis* rhs){
    id = rhs->getID();
    score = rhs->getScore();
    futureScore = rhs->getFutureScore();
    ptCoverage = &rhs->getCoverage();
    ptLastPhrasePair = rhs->getLastPhrasePair();
    lmHistory = rhs->getLMHistory();
    lmState = rhs->getLMState();
    lastCoveredSrcPos = rhs->getLastCoveredSrcPos();
    ptPrevHypothesis = rhs->getPrevLexicalHypothesis();
  }

  // ̕\
  void print() const {
   //  cerr << endl;
    cerr << "Hypothesis No." << id;
    cerr << " (from ";
    if(!ptPrevHypothesis)  cerr << "NULL";
    else cerr << ptPrevHypothesis->getID();
    cerr << ")" << endl;
    // 핢Pʒu
    cerr << "Coverage: ";    ptCoverage->print();
    cerr << "TotalScore: " << score << " + " << futureScore << " = " << score + futureScore << endl;

    // ꃂfp̗
    cerr << "LM History: ";
    for(unsigned i=StaticData::getInstance().getLanguageModel().getNGramOrder()-1; i>0; i--)
      cerr << StaticData::getInstance().getLanguageModel().getWord(lmHistory[i-1]) << " ";
    cerr << endl;

    const LexicalHypothesis* hypo = ptPrevHypothesis;
    while(hypo != NULL && hypo->getLastPhrasePair() != NULL){
      cerr << "Prev Phrase Pair: ";
      cerr << hypo->getLastPhrasePair()->getSrcPhrase().toString() << " <-> ";
      cerr << hypo->getLastPhrasePair()->getTgtPhrase().toString() << endl;
      hypo = hypo->getPrevLexicalHypothesis();
    }

    cerr << "Last Applied Phrase Pair: ";
    if(ptLastPhrasePair){
      cerr << ptLastPhrasePair->getSrcPhrase().toString() << " <-> ";
      cerr << ptLastPhrasePair->getTgtPhrase().toString();
    }
    cerr << endl;

    // Ōɖ|󂵂P
    cerr << "Last Translated Src Pos: " << lastCoveredSrcPos << endl << endl;
  }

  // ԍ(for Debug)
  const unsigned long getID() const { return id; }

  // RXg̕ԋp
  const float getScore() const { return score; }
  const float getFutureScore() const { return futureScore; }
  const float getEstimatedScore() const { return score + futureScore; }

  // recombinep
  const LMWordIDArray&     getLMHistory() const { return lmHistory; }
  const void*              getLMState()   const { return lmState; }
  const int                getLastCoveredSrcPos() const { return lastCoveredSrcPos; }
  const int                getLastCoveredSrcStart() const {
    const unsigned srcLength = (ptLastPhrasePair != NULL) ? ptLastPhrasePair->getSrcPhrase().size() : 1U;
    const int start = lastCoveredSrcPos - (int)srcLength + 1;
    return start;
  }
  const PhrasePair* const  getLastPhrasePair() const { return ptLastPhrasePair; }

  const Coverage&          getCoverage() const { return *ptCoverage; }
  const LexicalHypothesis* getPrevLexicalHypothesis() const { return ptPrevHypothesis; }


  // for Object Pool
  void                     setPrev(const LexicalHypothesis* prev){ ptPrevHypothesis = prev; }
  LexicalHypothesis*       getPrev() { return const_cast<LexicalHypothesis*>(ptPrevHypothesis); }

  // X^bNł̃\[gp
  bool operator>(const LexicalHypothesis& rhs) const {
    return getEstimatedScore() > rhs.getEstimatedScore();
  }  
};

unsigned long LexicalHypothesis::hypoNum = 0;

#endif
