#ifndef OBJECTPOOL_H_
#define OBJECTPOOL_H_

template <class T> class FixedObjectPool{
private:
  const unsigned capacity;
  T* pool;
  T* top;

  unsigned pushCount;
  unsigned popCount;

public:
  FixedObjectPool() : capacity(1000*1000U), pushCount(0U), popCount(0U){
    pool = new T[capacity];
    clear();
  }

  void clear(){
    pushCount = 0; popCount = 0;
    pool[0].setPrev(NULL);
    for(unsigned i=1; i<capacity; i++)
      pool[i].setPrev(&pool[i-1]);
    top = &pool[capacity-1];
  }

  T* pop(){
    popCount++;
    T* retVal = top;
    if(top == NULL){
      cerr << "No more Object in pool!!" << endl;
      printCounts();
      exit(1);
    }
    top = top->getPrev();

    return retVal;
  }

  // const T* top() const { return top; }
  void printCounts() const {
    cerr << "push: " << pushCount << endl;
    cerr << "pop : " << popCount << endl;
    cerr << "diff: " << popCount - pushCount << endl;
  }

  void push(T* object){
    pushCount++;
    object->setPrev(top);
    top = object;
  }
};

#endif /* OBJECTPOOL_H_ */
