#ifndef _MY_UTILITY_
#define _MY_UTILITY_

#include <string>
#include <deque>
#include <queue>
#include <algorithm>
#include <numeric>
#include <valarray>
#include <iomanip>
#include <sys/time.h>
#include "./Typedef.h"

using namespace std;

static float innerProduct(const WeightArray& weights, const ScoreArray& scores){
  return (weights*scores).sum();
}

template<class T> float getThresholdOfStackSize(const T& begin, const T& end, const unsigned size){
  priority_queue<float> scores;

  for(T iter = begin; iter != end; iter++)
    scores.push(iter->getEstimatedScore());
  for(unsigned i=0; i<size; i++) scores.pop();

  return scores.top();
}


template<class T, class S> unsigned pruneByThreshold(const T& begin, const T& end, const float threshold, S result){
  unsigned size = 0;
  for(T iter = begin; iter != end; iter++)
    if(iter->getEstimatedScore() > threshold){
      *result = &(*iter);
      size++;
      result++;
    }
  return size;
}

template<class T> bool hypoPtrScoreGreater(const T* a, const T* b){
  return a->getEstimatedScore() > b->getEstimatedScore();
}


static bool isSpace(const char c){
  if( c==' ' )
    return true;
  // else
  return false;

  // oO?
  // if( c==' ' || c=='\t' || c=='\n' ) return true;
}

static string strip(string str){
  while( isSpace(str[0]) )
    str.erase(0,1);
  while( isSpace(str.length()-1) )
    str.erase(str.length()-1, 1);

  return str;
}

static StringArray split(const string& str, const string& delimiter=" "){
  StringArray elems;
  const unsigned delimiterLength = delimiter.length();

  int currentPos = 0;
  int prevPos = 0;
  while( (currentPos=str.find(delimiter, currentPos)) != string::npos ){
    elems.push_back( str.substr(prevPos,currentPos-prevPos) );
    if(elems.back().length() == 0) elems.pop_back();
    currentPos += delimiterLength;
    prevPos = currentPos;
  }
  elems.push_back( str.substr(prevPos) );
  if(elems.back().length() == 0) elems.pop_back();

  return elems;
}

static double diffTimeVal(const timeval& s, const timeval& e){
  return ( (e.tv_sec-s.tv_sec) + (e.tv_usec-s.tv_usec)*1e-6 );
}

static double tvToDouble(const timeval& tv){
  return tv.tv_sec + tv.tv_usec*1e-6;
}


#endif
